import { useState, useEffect } from "react"
import { Link } from "react-router-dom"
import { Channels, RecentChannels, User, Messages } from "../types"
import MessageComponent from "../components/MessageComponent"
import axios from "axios"

export default function Home() {
    const [user, setUser] = useState<User>();
    const [channels, setChannels] = useState<RecentChannels>();
    const [messages , setMessages] = useState<Messages>();
    const [searchedChannels, setSearchedChannels] = useState<Channels>([]);
    const [search, setSearch] = useState<string>("");

    useEffect(() => {
        const token = localStorage.getItem("token")
        if (token) {
            axios
                .post("/api/auth/me", {
                    token: token
                })
                .then((res) => {
                    setUser(res.data)
                })
                .catch((err) => {
                    console.error(err.response)
                })
        } 

        axios
            .get("/api/activechannels")
            .then((res) => {channels
                setChannels(res.data)
            })
            .catch((err) => {
                console.error(err.response)
            })
        
        axios
            .get("/api/lastmessages")
            .then((res) => {
                setMessages(res.data)
            })
            .catch((err) => {
                console.error(err.response)
            }
        )
    }, [])

    useEffect(() => {
        const id = setInterval(() => {
            axios
                .get("/api/activechannels").then((res) => {
                    setChannels(res.data)
                }
            )

            axios
                .get("/api/lastmessages").then((res) => {
                    setMessages(res.data)
                }
            )

        }, 2000)

        return () => { clearInterval(id) }
    }, [])

    useEffect(() => {
        if (!search) {
            setSearchedChannels([])
            return
        }
        axios
            .get(`/api/searchchannels?search=${search}`).then((res) => {
                setSearchedChannels(res.data)
            }
            ).catch((err) => {
                console.error(err.response)
            })
    }, [search])

    return (
        <div>
            <h1>Home</h1>
            {user ? (
                <div>
                    <p>Welcome {user.username}</p>
                    <button onClick={() => {
                        localStorage.removeItem("token")
                        window.location.reload()
                    }}>Logout</button>
                    <Link to="/create-channel">Create Channel</Link>
                </div>
            ) : (
                <div>
                    <Link to="/login">Login</Link>
                    <Link to="/register">Register</Link>
                </div>
            )}
            <h2>Recent active channels</h2>
            <ul>
                {channels?.map((channel) => (
                    <li key={channel.id}>
                        <Link to={`/c/${channel.name}`}>{channel.name}</Link>
                    </li>
                ))}
            </ul>
            <h2>Search channels</h2>
            <input
                type="text"
                placeholder="Search channels"
                value={search}
                onChange={(e) => setSearch(e.target.value)}
            />
            <ul>
                {searchedChannels?.map((channel) => (
                    <li key={channel.id}>
                        <Link to={`/c/${channel.name}`}>{channel.name}</Link>
                    </li>
                ))}
            </ul>
            <Link to={'/channels'}>All channels</Link>
            <h2>Last messages</h2>
            <ul>
                {messages?.map((message) => (
                    <MessageComponent
                        key={message.id}
                        message={message}
                        user={user}
                        channel={undefined}
                        deleteMessage={undefined}
                    />
                ))}
            </ul>
            <img src="osaka_arch.png" alt="osaka" className="osaka"/>
        </div>
    )
}